#pragma once

#include "codec.hh"
#include "params.hh"

#include "osipparser2/osip_parser.h"

class Base_Type;
class TTCN_Typedescriptor_t;
class TTCN_Buffer;

namespace LibSip__SIPTypesAndValues {
  class Request;
  class HostPort;
  class Authorization;
  class From;
  class To;
  class Via;
}

namespace LibSip__Common {
  class SemicolonParam__List;
  class AmpersandParam__List;
}
// class OPTIONAL<LibSip__Common::SemicolonParam__List>;
// class OPTIONAL<LibSip__Common::AmpersandParam__List>;

class sip_codec_request: public codec <
LibSip__SIPTypesAndValues::Request,
LibSip__SIPTypesAndValues::Request>
{
public:
  explicit sip_codec_request() : codec<LibSip__SIPTypesAndValues::Request, LibSip__SIPTypesAndValues::Request>() { };
  virtual ~sip_codec_request() { };

  virtual int encode (const LibSip__SIPTypesAndValues::Request& msg, OCTETSTRING& data);
  virtual int decode (const OCTETSTRING& data, LibSip__SIPTypesAndValues::Request& msg, params* p_params = NULL);
private: //! \todo Move this section into a sip_codec_helper class, need to decode Response too
  void fill_message_body(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::Request& p_request);
  void fill_payload(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::Request& p_request);
  void fill_uri(LibSip__SIPTypesAndValues::SipUrl& p_sip_url, const osip_uri_t* p_uri);
  void fill_request_line(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::Request& msg);
  void fill_headers(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::Request& msg);


  void fill_accept_header(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::Accept& p_accept_header);
  void fill_accept_contact_header(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::AcceptContact& p_accept_contact_header);
  void fill_accept_encoding_header(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::AcceptEncoding& p_accept_encoding_header);
  void fill_accept_language_header(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::AcceptLanguage& p_accept_language_header);
  void fill_alert_info_header(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::AlertInfo& p_alert_info_header);
  void fill_allow_header(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::Allow& p_allow_header);
  void fill_allow_events_header(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::AllowEvents& p_allow_events_header);
  void fill_authorization_header(const osip_message_t* p_sip_message, LibSip__SIPTypesAndValues::Authorization& p_authorization_header);
  void fill_from_header(const osip_from_t* p_sip_from, LibSip__SIPTypesAndValues::From& p_from_header);
  void fill_to_header(const osip_to_t* p_sip_to, LibSip__SIPTypesAndValues::To& p_to_header);
  void fill_via_header(const osip_message_t* p_sip_via_list, LibSip__SIPTypesAndValues::Via& p_via_header);
  
  void fill_host_port(const char* p_host, const char* p_port, LibSip__SIPTypesAndValues::HostPort& p_host_port);
  void fill_semi_colon_params(const osip_list_t& p_sip_list, OPTIONAL<LibSip__Common::SemicolonParam__List>& p_list);
  void fill_ampersand_params(const osip_list_t& p_sip_list, OPTIONAL<LibSip__Common::AmpersandParam__List>& p_list);

}; // End of class sip_codec_request
