/*!
 * \file      sip_layer.hh
 * \brief     Header file for ITS HTTP protocol layer.
 * \author    ETSI STF549
 * \copyright ETSI Copyright Notification
 *            No part may be reproduced except as authorized by written permission.
 *            The copyright and the foregoing restriction extend to reproduction in all media.
 *            All rights reserved.
 * \version   0.1
 */
#pragma once

#include <memory>

#include "t_layer.hh"

#include "sip_codec_register_request.hh"

namespace LibSip__Interface {
  class SipPort; //! Forward declaration of TITAN class
}

namespace LibSip__SIPTypesAndValues {
  class REGISTER__Request; //! Forward declaration of TITAN class
}

class OCTETSTRING; //! Forward declaration of TITAN class

/*!
 * \class sip_layer
 * \brief  This class provides a factory class to create an sip_layer class instance
 */
class sip_layer : public t_layer<LibSip__Interface::SipPort> {
  params _params;

public: //! \publicsection
  /*!
   * \brief Specialised constructor
   *        Create a new instance of the sip_layer class
   * \param[in] p_type \todo
   * \param[in] p_param \todo
   */
  sip_layer() : t_layer<LibSip__Interface::SipPort>(), _params() {};
  /*!
   * \brief Specialised constructor
   *        Create a new instance of the sip_layer class
   * \param[in] p_type \todo
   * \param[in] p_param \todo
   */
  sip_layer(const std::string& p_type, const std::string& p_param);
  /*!
   * \brief Default destructor
   */
  virtual ~sip_layer() { };

  /*!
   * \virtual
   * \fn void send_data(OCTETSTRING& data, params& p_params);
   * \brief Send bytes formated data to the lower layers
   * \param[in] p_data The data to be sent
   * \param[in] p_params Some parameters to overwrite default value of the lower layers parameters
   */
  virtual void send_data(OCTETSTRING& data, params& p_params);
  /*!
   * \virtual
   * \fn void receive_data(OCTETSTRING& data, params& p_params);
   * \brief Receive bytes formated data from the lower layers
   * \param[in] p_data The bytes formated data received
   * \param[in] p_params Some lower layers parameters values when data was received
   */
  virtual void receive_data(OCTETSTRING& data, params& info);
}; // End of class sip_layer

